export declare class TreeNode<V = any> {
    /**
     * The constructor function initializes a TreeNode object with a key, optional value, and optional
     * children.
     * @param {string} key - A string representing the key of the tree node.
     * @param {V} [value] - The `value` parameter is an optional parameter of type `V`. It represents the
     * value associated with the node. If no value is provided, it defaults to `undefined`.
     * @param {TreeNode<V>[]} [children] - The `children` parameter is an optional array of `TreeNode<V>`
     * objects. It represents the child nodes of the current node. If no children are provided, the
     * default value is an empty array.
     */
    constructor(key: string, value?: V, children?: TreeNode<V>[]);
    protected _key: string;
    /**
     * The function returns the value of the protected variable _key.
     * @returns The value of the `_key` property, which is a string.
     */
    get key(): string;
    /**
     * The above function sets the value of a protected variable called "key".
     * @param {string} value - The value parameter is a string that represents the value to be assigned
     * to the key.
     */
    set key(value: string);
    protected _value?: V | undefined;
    /**
     * The function returns the value stored in a variable, or undefined if the variable is empty.
     * @returns The value of the variable `_value` is being returned.
     */
    get value(): V | undefined;
    /**
     * The function sets the value of a variable.
     * @param {V | undefined} value - The parameter "value" is of type "V | undefined", which means it
     * can accept a value of type "V" or it can be undefined.
     */
    set value(value: V | undefined);
    protected _children?: TreeNode<V>[] | undefined;
    /**
     * The function returns an array of TreeNode objects or undefined.
     * @returns The `children` property is being returned. It is of type `TreeNode<V>[] | undefined`,
     * which means it can either be an array of `TreeNode<V>` objects or `undefined`.
     */
    get children(): TreeNode<V>[] | undefined;
    /**
     * The function sets the value of the children property of a TreeNode object.
     * @param {TreeNode<V>[] | undefined} value - The value parameter is of type TreeNode<V>[] |
     * undefined. This means that it can accept an array of TreeNode objects or undefined.
     */
    set children(value: TreeNode<V>[] | undefined);
    /**
     * The function `addChildren` adds one or more child nodes to the current node.
     * @param {TreeNode<V> | TreeNode<V>[]} children - The `children` parameter can be either a single
     * `TreeNode<V>` object or an array of `TreeNode<V>` objects.
     */
    addChildren(children: TreeNode<V> | TreeNode<V>[]): void;
    /**
     * The function `getHeight()` calculates the maximum depth of a tree structure by performing a
     * breadth-first search.
     * @returns the maximum depth or height of the tree.
     */
    getHeight(): number;
}
